package net.cephas.james.mailets;


import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.mail.MessagingException;

import org.apache.mailet.GenericMailet;
import org.apache.mailet.Mail;
import org.apache.mailet.MailAddress;
import org.apache.mailet.MailetConfig;
import org.jivesoftware.smack.Chat;
import org.jivesoftware.smack.XMPPConnection;
import org.jivesoftware.smack.XMPPException;


/**
 * Mail2XMPPMailet is an Apache James Mailet that enables users to receive
 * an instant message that can contain a configurable amount of information
 * about the email received via an XMPP compatible IM account (ie: Google Talk)
 * using the Smack client library. 
 * 
 * By default messages are transformed by the SimpleSummarizer, which sends the
 * from name, the from email address and the subject.  The mailet can be
 * configured to use a different summarizer by specifying the fully qualified
 * name of a class that implements the MailSummarizer interface in the
 * configuration of the mailet by using a <code>summarizer</code> element. If there 
 * are errors locating and / or creating an instance of the class specified by 
 * <code>summarizer</code> element, the Mail2XMPPMailet will a) write a message
 * to the mailet log file and b) continue to function but will use the 
 * SimpleSummarizer.
 * 
 * The Mail2XMPPMailet requires the following mailet configuration entries:
 * <ul>
 * <li>xmppserver: which should be in the form of 
 * 		<code>hostname:port:servicename</code>. For example, 
 * 		<code>talk.google.com:5222:gmail.com</code>.  Both this entry and the next
 * refer to the XMPP account that forwards the messages.  This account most 
 * likely should be an account associated with a computer / system and not
 * a person.</li>
 * <li>xmpplogin: which should be in the form of
 * 		<code>username:password</code>.</li>
 * <li>recipients: which is a comma delimited list of recipients, each of
 * which is an equal sign (=) delimited list of email addresses and 
 * matching XMPP usernames. For example, to have an XMPP message sent to
 * ajohnson@jabber.org if an email is received to ajohnson@cephas.net, you'd
 * have:
 * 		<code>ajohnson@cephas.net=ajohnson@jabber.org,
 * 	someone_else@gmail.com=someone_else@gmail.com</code>
 * </li>
 * As mentioned above, the Mail2XMPPMailet can be optionally configured with
 * a <code>summarizer</code> configuration entry. If a different <code>summarizer</code>
 * class is used, the class must exist in either $JAMES\apps\james\SAR-INF\lib\ or
 * in $JAMES\apps\james\SAR-INF\classes.
 *
 * @author Aaron Johnson
 */
public class Mail2XMPPMailet extends GenericMailet {

    private boolean inService = true;
    
    private String xmpphost;
    private int xmppport;
    private String xmppservice;
    private String xmppusername;
    private String xmpppassword;
    
    private MailSummarizer summarizer; 
    private Map recipients;
    
    public void init(MailetConfig config) throws MessagingException {
        super.init(config);
        
        // attempt to initialize the smack client, both configs are colon delimited
        String[] xmppserver = getInitParameter(Constants.CONFIG_XMPPSERVER).split(":");
        String[] xmpplogin = getInitParameter(Constants.CONFIG_XMPPLOGIN).split(":");
        try {
            xmpphost = xmppserver[0];
            xmppport = Integer.parseInt(xmppserver[1]);
            xmppservice = xmppserver[2];
        } catch (IndexOutOfBoundsException iob) {
            inService = false;
            log("The <xmppserver> configuration element must contain three (3) items " +
            		"separated by a colon: host, port and service name.");
        } catch (NumberFormatException nfe) {
            inService = false;
            log("The port you provided in the <xmppserver> configuration is not a valid number.");
        }

        try {
            xmppusername = xmpplogin[0];
            xmpppassword = xmpplogin[1];
        } catch (IndexOutOfBoundsException iob) {
            inService = false;
            log("The <xmpplogin> configuration element must contain two (2) items " +
            		"separated by a colon: username and password.");
        }

        
        try {
            XMPPConnection con = new XMPPConnection(xmpphost, xmppport, xmppservice);
            if (!con.isConnected()) {
                inService = false;
                log("Mail2XMPPMailet was not able to make a connection to the XMPP server.");
                
            } else {
                con.login(xmppusername, xmpppassword);
                if (!con.isAuthenticated()) {
                    inService = false;
                    log("The <xmpplogin> information you provided is not valid.");
                }
            }
        } catch (XMPPException xe) {
            inService = false;
            log("There was an error connecting to the XMPP service: " + xe.toString());
        }
        
        // attempt to initalize the recipients
        String[] configrecipients = getInitParameter(Constants.CONFIG_RECIPIENTS).split(",");
        if (configrecipients != null && configrecipients.length > 0) {
            recipients = new HashMap();
            for (int i=0; i<configrecipients.length; i++) {
                String[] user = configrecipients[i].split("=");
                recipients.put(user[0], user[1]);
            }
        } else {
            inService = false;
            log("There were no recipients configured for the Mail2XMPPMailet.");
        }
        
        
        // attempt to initialize the summarizer 
        String classname = getInitParameter(Constants.CONFIG_SUMMARIZER);
        if (classname != null) {
            try {
                Class clazz = this.getClass().getClassLoader().loadClass(classname);
                summarizer = (MailSummarizer)clazz.newInstance();
            } catch (ClassNotFoundException e) {
                log("The class '" + classname + "' could not be found.");
            } catch (InstantiationException e) {
                log("The class '" + classname + "' could not be instantiated.");
            } catch (IllegalAccessException e) {
                log("The class '" + classname + "' constructor could not be invoked.");
            }
        } else {
            summarizer = new SimpleSummarizer();
        }
    }

    public void service(Mail mail) {
     
        if (inService) {
            Collection recs = mail.getRecipients();
            List imRecipients = new ArrayList();
            for (Iterator i = recs.iterator(); i.hasNext();) {
                MailAddress m = (MailAddress)i.next();
                String email = m.toString();
                if (recipients.containsKey(email)) {
                    imRecipients.add((String)recipients.get(email));
                }
            }
            
            if (imRecipients.size() > 0) {
                String message = null;
                
                try {
                    // if something goes wrong while constructing the message
                    // catch the error, log the error but then don't 
                    // attempt to send the chat message
                    message = summarizer.getSummary(mail);
                } catch (MessagingException me) {
                    log("An error occurred while attempting to summarize the email message: " +
                    	me.toString());
                }
                
                if (message != null) {
                    XMPPConnection con = null;
                    try {
                        con = new XMPPConnection(xmpphost, xmppport, xmppservice);
                        con.login(xmppusername, xmpppassword);
                        for (int i=0; i<imRecipients.size(); i++) {
                        	Chat c = con.createChat((String)imRecipients.get(i));
                        	c.sendMessage(message);

                        }
                    } catch (XMPPException e) {
                        log("An error occurred while sending the following message: '" + message +
                                "'. Error message: " + e.toString());
                    } finally {
                        if (con != null && con.isConnected()) {
                            con.close();   
                        }
                    }
                    
                }
                
            }
        }
        
    }
}
